<?php

/**
 * Helper for the Contextly service integration.
 */
class Contextly {

  protected function __construct() {}

  static public function getInstance() {
    static $instance;

    if (!isset($instance)) {
      $instance = new self();
    }

    return $instance;
  }

  /**
   * Sends the node to Contextly.
   *
   * @param object $node
   */
  public function putNode($node) {
    $this->putNodeContent($node);
    if (module_exists('taxonomy')) {
      $this->putNodeTags($node);
    }
  }

  /**
   * Sends the node text content and meta-information to Contextly.
   *
   * @param object $node
   */
  protected function putNodeContent($node) {
    // Build absolute node URL.
    $node_url = url('node/' . $node->nid, array('absolute' => TRUE));

    $api = ContextlyApi::getInstance();

    // Check if post has been saved to Contextly earlier.
    $contextly_post = $api->api('posts', 'get')
      ->param('page_id', $node->nid)
      ->get();

    $user = user_load($node->uid);

    // Build node content. Mark the node to let the other modules and preprocess
    // function know about the rendering mode. Remove the mark after getting
    // HTML.
    $node->contextly_view_mode = TRUE;
    $content = node_view($node);
    unset($node->contextly_view_mode);

    $post_data = array(
      'post_id' => $node->nid,
      'post_title' => $node->title,
      'post_date' => format_date($node->changed, 'custom', 'Y-m-d H:i:s'),
      'post_status' => $node->status ? 'publish' : 'draft',
      'post_type' => $node->type,
      'post_content' => $content,
      'url' => $node_url,
      'author_id' => $node->uid,
      'post_author' => $user->name,
    );
    $api
      ->api('posts', 'put')
      ->extraParams($post_data);
    if (isset($contextly_post->entry)) {
      $api->param('id', $contextly_post->entry->id);
    }

    $response = $api->get();
    if (empty($response->success)) {
      throw new ContextlyApiException(t('Unable to send content of the node #@nid to the Contextly.', array(
        '@nid' => $node->nid,
      )), $response);
    }
  }

  /**
   * Extracts tags list attached to the node that should be sent to Contextly.
   *
   * @param object $node
   *
   * @return array
   *
   * @todo take care about making tags unique? +D7
   */
  protected function extractNodeTags($node) {
    $tags = array();

    // Use either all available fields or selected fields only.
    if (variable_get('contextly_all_tags', TRUE)) {
      $all = TRUE;
      $vids = array();
    }
    else {
      $all = FALSE;
      $vids = variable_get('contextly_tags__' . $node->type, array());
    }

    if (!empty($node->taxonomy)) {
      foreach ($node->taxonomy as $term) {
        if ($all || !empty($vids[$term->vid])) {
          $tags[] = $term->name;
        }
      }
    }

    return $tags;
  }

  /**
   * Sends node tags to the Contextly.
   *
   * @param object $node
   */
  protected function putNodeTags($node) {
    $api = ContextlyApi::getInstance();

    // Remove existing tags first, if any.
    // TODO: Handle pagination of the request.
    $post_tags = $api->api('poststags', 'list')
      ->searchParam('post_id', ContextlyApi::SEARCH_TYPE_EQUAL, $node->nid)
      ->get();
    if (!empty($post_tags->list)) {
      foreach ($post_tags->list as $tag) {
        $api->api('poststags', 'delete')
          ->param('id', $tag->id)
          ->get();
        // TODO Check response.
      }
    }

    // Save new tags.
    $tags = $this->extractNodeTags($node);
    // TODO WP Plugin sends only 3 first tags. Why?
    foreach ($tags as $tag) {
      $tag_data = array(
        'post_id' => $node->nid,
        'name' => $tag,
      );

      $response = $api->api('poststags', 'put')
        ->extraParams($tag_data)
        ->get();
      if (empty($response->success)) {
        throw new ContextlyApiException(t('Unable to save tag @name', array('@name' => $tag)), $response);
      }
    }
  }

  /**
   * Loads page widgets from Contextly.
   *
   * @param int $nid
   * @param int $admin
   *
   * @return object
   *
   * @throws ContextlyApiException
   */
  protected function getPageWidgets($nid, $admin) {
    $response = ContextlyApi::getInstance()
      ->api('pagewidgets', 'get')
      ->param('page_id', $nid)
      ->param('admin', $admin)
      ->get();
    if (!empty($response->success) || !empty($response->entry)) {
      return $response->entry;
    }
    else {
      throw new ContextlyApiException(t('Unable to load page widgets.'), $response);
    }
  }

  /**
   * Returns JS settings for the node edit form.
   *
   * @param $node
   *
   * @return array
   */
  public function buildEditorSettings($node) {
    $api_key = variable_get('contextly_api_key', array());
    $site_path = '';
    if (isset($api_key['appID'])) {
      $site_path = $api_key['appID'];
    }
    return array(
      'contextlyEditor' => array(
        'sitePath' => $site_path,
        'popupServerUrl' => _contextly_server_url('popup'),
        'nid' => $node->nid,
        'settingsURL' => url('contextly-ajax/node-edit/' . $node->nid),
      ),
    );
  }

  /**
   * Loads required settings from Contextly for the node edit form.
   *
   * @param stdClass $node
   *
   * @return array
   */
  public function loadEditorSettings($node) {
    $snippet = NULL;
    $sidebars = array();

    $widgets = $this->getPageWidgets($node->nid, 1);
    if (!empty($widgets->snippets) && is_array($widgets->snippets)) {
      $snippet = reset($widgets->snippets);
    }
    if (!empty($widgets->sidebars)) {
      foreach ($widgets->sidebars as $sidebar) {
        if (!empty($sidebar->id)) {
          $sidebars[$sidebar->id] = $sidebar;
        }
      }
    }

    return array(
      'snippet' => $snippet,
      'sidebars' => $sidebars,
    );
  }

  /**
   * Returns JS settings required for the widgets.
   *
   * @param object $node
   *
   * @return array
   */
  public function buildWidgetsSettings($node) {
    $api_server = _contextly_server_url('api');
    $api_key = variable_get('contextly_api_key', array());
    $site_path = '';
    if (isset($api_key['appID'])) {
      $site_path = $api_key['appID'];
    }
    return array(
      'contextlyWidgets' => array(
        'nid' => $node->nid,
        'version' => CONTEXTLY_PLUGIN_VERSION,
        'mode' => variable_get('contextly_mode', ''),
        'sitePath' => $site_path,
        'providerURL' => $api_server . 'easy_xdm/cors/index.html',
        'settingsURL' => $api_server . 'pagewidgets/get/',
        'pageEventsURL' => $api_server . 'siteevents/put/',
      ),
    );
  }

}

/**
 * Custom exception class to distinguish it from other kinds of exceptions.
 */
class ContextlyException extends Exception {}
