(function ($) {

  // JSON RegExp, required for the parseJSON().
  var rvalidchars = /^[\],:{}\s]*$/,
    rvalidbraces = /(?:^|:|,)(?:\s*\[)+/g,
    rvalidescape = /\\(?:["\\\/bfnrt]|u[\da-fA-F]{4})/g,
    rvalidtokens = /"[^"\\\r\n]*"|true|false|null|-?(?:\d\d*\.|)\d+(?:[eE][\-+]?\d+|)/g;

  Drupal.contextly = Drupal.contextly || {};

  /**
   * Common features for the links/sidebars editing.
   */
  Drupal.contextly.editor = {
    isLoaded: false,
    isLoading: false,

    /**
     * Port of the jQuery 1.8 function.
     */
    parseJSON: function (data) {
      if (!data || typeof data !== "string") {
        return null;
      }

      // Make sure leading/trailing whitespace is removed (IE can't handle it).
      data = jQuery.trim(data);

      // Attempt to parse using the native JSON parser first.
      if (window.JSON && window.JSON.parse) {
        return window.JSON.parse(data);
      }

      // Make sure the incoming data is actual JSON
      // Logic borrowed from http://json.org/json2.js
      if (rvalidchars.test(data.replace(rvalidescape, "@")
        .replace(rvalidtokens, "]")
        .replace(rvalidbraces, ""))) {

        return ( new Function("return " + data) )();

      }
      $.error("Invalid JSON: " + data);
    },

    open: function (type) {
      var func = '';
      switch (type) {
        case 'general':
          func = this.openGeneralEditor;
          break;

        case 'link':
          func = this.openLinkEditor;
          break;

        case 'sidebar':
          func = this.openSidebarEditor;
          break;

        default:
          return;
      }

      // Remove the type from the arguments.
      var args = Array.prototype.slice.call(arguments, 1);

      // When loading is still in progress, add function to the queue.
      if (!this.isLoaded) {
        // Load settings if not yet started.
        if (!this.isLoading) {
          this.loadSettings();
        }
      }
      else {
        func.apply(this, args);
      }
    },

    openGeneralEditor: function () {
      var s = this.getSettings();

      var url = s.popupServerUrl
        + 'sites/' + encodeURIComponent(s.sitePath) + '/'
        + '?page_id=' + encodeURIComponent(s.nid)
        + '&edit_snippet_id=' + (s.snippet && s.snippet.id ? encodeURIComponent(s.snippet.id) : '');

      this.openPopupWindow(url, function (response) {
        if (response.data && response.data.entry && response.data.entry.id) {
          // Update settings.
          this.setSnippet(response.data.entry);
        }
        else {
          // TODO Error message to the user.
        }
      }, this);
    },

    openLinkEditor: function (text, callback, context) {
      var s = this.getSettings();

      var url = s.popupServerUrl
        + 'sites/' + encodeURIComponent(s.sitePath) + '/'
        + '?page_id=' + encodeURIComponent(s.nid)
        + '&edit_snippet_id=' + (s.snippet && s.snippet.id ? encodeURIComponent(s.snippet.id) : '')
        + '&tinymce_link_text=' + encodeURIComponent(text);

      this.openPopupWindow(url, callback, context);
    },

    openSidebarEditor: function (sidebarId, callback, context) {
      var s = this.getSettings();

      var url = s.popupServerUrl
        + 'sites/' + encodeURIComponent(s.sitePath) + '/sidebar/'
        + '?page_id=' + encodeURIComponent(s.nid)
        + '&sidebar_id=' + (sidebarId ? encodeURIComponent(sidebarId) : '');

      this.openPopupWindow(url, function (response) {
        if (response.data && response.data.entry && response.data.entry.id) {
          // Run external callback first.
          var callbackResult = callback.call(context, response);
          if (callbackResult === false) {
            // Callback interrupted execution, just exit.
            return;
          }

          // Update global settings.
          this.setSidebar(response.data.entry);
        }
        else {
          // TODO Error message to the user.
        }
      }, this);
    },

    openPopupWindow: function (url, callback, context) {
      url += '#easyXDM_linker_channel_provider';
      window.open(url, 'contextlyapp', 'width=1150,height=600,resizable=1,scrollbars=1,menubar=1');

      if (typeof callback === 'function') {
        var s = this.getSettings();
        var socket = new easyXDM.Socket({
          onMessage: function (data, origin) {
            socket.destroy();

            var response = null;
            try {
              response = Drupal.contextly.editor.parseJSON(data);
            }
            catch (e) {
              // Just suppress the exception.
            }

            if (response && response.status === "ok") {
              callback.call(context, response);
            }
            else {
              // TODO Message to the user.
            }
          },
          channel: "linker_channel",
          remote: s.popupServerUrl + "resources/html/remote.html"
        });
      }
    },

    loadSettings: function () {
      var s = this.getSettings();

      this.isLoaded = false;
      this.isLoading = true;

      var self = this;
      $.ajax({
        url: s.settingsURL,
        dataType: 'json',
        success: function(data) {
          self.onSettingsLoadingSuccess(data);
        },
        error: function() {
          self.onSettingsLoadingFailure();
        },
        cache: false
      });

      this.fireEvent('contextlySettingsLoading');
    },

    getSettings: function () {
      return $.extend({
        nid: 0,
        sitePath: '',
        popupServerUrl: 'http://api.contextly.com/',
        settingsURL: null,
        sidebars: {},
        snippet: null
      }, Drupal.settings.contextlyEditor);
    },

    onSettingsLoadingSuccess: function (data) {
      // Set flags.
      this.isLoaded = true;
      this.isLoading = false;

      // Put new data into the settings.
      $.extend(Drupal.settings.contextlyEditor, data);

      // Let the other scripts to know about loaded settings.
      this.fireEvent('contextlySettingsLoaded');
    },

    onSettingsLoadingFailure: function () {
      this.fireEvent('contextlySettingsFailed');
    },

    fireEvent: function (type) {
      // Remove the type of event.
      var args = Array.prototype.slice.call(arguments, 1);

      $(window).triggerHandler(type, args);
    },

    setSidebar: function(entry) {
      Drupal.settings.contextlyEditor.sidebars[entry.id] = entry;
      this.fireEvent('contextlySettingsUpdated', 'sidebar', entry.id);
    },

    setSnippet: function(entry) {
      Drupal.settings.contextlyEditor.snippet = entry;
      this.fireEvent('contextlySettingsUpdated', 'snippet', entry.id);
    }
  };

  /**
   * Behavior to load the settings on the DOM ready.
   */
  Drupal.behaviors.contextlyEditor = function (context) {
    var markerClass = 'contextly-editor-processed';
    $('body:not(.' + markerClass + ')', context)
      .addClass(markerClass)
      .each(function () {
        Drupal.contextly.editor.loadSettings();
      });
  };

})(jQuery);
